
// Simple (not optimized) implementation of Runge-Kutta 1 (Euler)
// Needs linking with math library (GCC option -lm)
// Copyright: (c) Petr Peringer, FIT TU Brno, 2018

#include <stdio.h>
#include <math.h>               // sin() - used only for error computation

// System: circle test
#define SIZE 2                  // state vector size of the system
// "Dynamic section" = continuous system description
// Input:  t = time of step start,
//         y = state vector = integrator outputs
// Output: yin = vector of derivatives = integrators input values
void f(double t, double *y, double *yin)
{
    yin[0] = y[1];
    yin[1] = -y[0];
}

// Euler method
// Parameters: t = start time,
//             h = step size,
//             y = state vector
// Output:  new state in y
// Warning: it does not update time
void RK1step(double t, double h, double *y)
{
    double yin[SIZE];           // integrator input = derivative

    f(t, y, yin);               // yin = f(t, y(t))

    for (int i = 0; i < SIZE; i++) {
        y[i] = y[i] + h * yin[i]; // y(t+h) = y(t) + h * f(t, y(t))
    }
    // Return: y = new state at time t+h
}

// Experiment control
int main()
{
    double y[SIZE] = { 0, 1 };
    double stepsize = 0.01;

    double t = 0;
    while (t < 20) {
        printf("%8.3f % -11g % -11g  % e\n", t, y[0], y[1], y[0] - sin(t));
        RK1step(t, stepsize, y);        // make step
        t += stepsize;          // advance the simulation time
    }
    printf("%8.3f % -11g % -11g  % e\n", t, y[0], y[1], y[0] - sin(t));
    return 0;
}
